@@ -85,8 +85,15 @@ process_initialize <- function(self, private, command, args,
   pty_options$cols <- as.integer(pty_options$cols)
   pty_options <- pty_options[names(def)]
 
-  command <- enc2utf8(command)
-  args <- enc2utf8(args)
+  command <- enc2path(command)
+  args <- enc2path(args)
+
+  wd <- wd %||% getwd()
+  if (!is.null(wd)) {
+    # check is needed if the current working directory does not exist
+    # `mustWork = FALSE` is needed if the supplied wd does not exist
+    wd <- enc2path(normalizePath(wd, mustWork = FALSE))
+  }
 
   private$command <- command
   private$args <- args
