@@ -179,7 +179,7 @@ SEXP processx_connection_read_lines(SEXP con, SEXP nlines) {
   result = PROTECT(allocVector(STRSXP, lines_read + eof));
   for (l = 0, newline = -1; l < lines_read; l++) {
     eol = processx__find_newline(ccon, newline + 1);
-    slashr = ccon->utf8[eol - 1] == '\r';
+    slashr = eol > 0 && ccon->utf8[eol - 1] == '\r';
     SET_STRING_ELT(
       result, l,
       mkCharLenCE(ccon->utf8 + newline + 1,
@@ -617,11 +617,12 @@ ssize_t processx_c_connection_write_bytes(
   sigaction(SIGPIPE, &new_handler, &old_handler );
 
   ssize_t ret = write(ccon->handle, buffer, nbytes);
+  int err = errno;
 
   sigaction(SIGPIPE, &old_handler, NULL );
 
   if (ret == -1) {
-    if (errno == EAGAIN || errno == EWOULDBLOCK) {
+    if (err == EAGAIN || err == EWOULDBLOCK) {
       return 0;
     } else {
       R_THROW_SYSTEM_ERROR("Cannot write connection");
@@ -1607,4 +1608,27 @@ int processx__connection_schedule_destroy(processx_connection_t *ccon) {
 
 #endif
 
+#ifdef _WIN32
+
+SEXP processx_is_valid_fd(SEXP fd) {
+  int cfd = INTEGER(fd)[0];
+  HANDLE hnd = (HANDLE) _get_osfhandle(cfd);
+  int valid =
+    hnd != INVALID_HANDLE_VALUE &&
+    hnd != NULL &&
+    hnd != (HANDLE) (-2);
+  return ScalarLogical(valid);
+}
+
+#else
+
+SEXP processx_is_valid_fd(SEXP fd) {
+  int cfd = INTEGER(fd)[0];
+  errno = 0;
+  int valid = fcntl(cfd, F_GETFD) != -1 || errno != EBADF;
+  return ScalarLogical(valid);
+}
+
+#endif
+
 #undef PROCESSX_CHECK_VALID_CONN
