@@ -1,15 +1,33 @@
 
 #include "../processx.h"
 
+#include <pthread.h>
+
 extern processx__child_list_t *child_list;
 
+static struct sigaction old_sig_handler = {{ 0 }};
+int processx__notify_old_sigchld_handler = 0;
+pthread_t processx__main_thread = { 0 };
+
 void processx__sigchld_callback(int sig, siginfo_t *info, void *ctx) {
+  /* This might be called on another thread, if the main thread blocks
+     the signal temporarily, so we forward it to the main thread.
+     It is OK to call pthread_self() and pthread_kill() in the signal
+     handler, according to POSIX.1-2008 TC1.
+     https://man7.org/linux/man-pages/man7/signal-safety.7.html */
+  if (pthread_self() != processx__main_thread) {
+    pthread_kill(processx__main_thread, SIGCHLD);
+    return;
+  }
+
+  /* This should really not happem, but just in case. */
   if (sig != SIGCHLD) return;
 
   /* While we get a pid in info, this is basically useless, as
      (on some platforms at least) a single signal might be delivered
-     for multiple children exiting around the same time. So we need to
-     iterate over all children to see which one has exited. */
+     for multiple children exiting around the same time. For example this
+     happens if multiple SIGCHLD signals arrive while SIGCHLD is blocked.
+     So we need to iterate over all children to see which one has exited. */
 
   processx__child_list_t *ptr = child_list->next;
   processx__child_list_t *prev = child_list;
@@ -67,23 +85,38 @@ void processx__sigchld_callback(int sig, siginfo_t *info, void *ctx) {
       ptr = next;
     }
   }
-}
 
-/* TODO: use oldact */
+  if (processx__notify_old_sigchld_handler) {
+    if (old_sig_handler.sa_handler != SIG_DFL &&
+        old_sig_handler.sa_handler != SIG_IGN &&
+        old_sig_handler.sa_handler != NULL) {
+      if (old_sig_handler.sa_flags | SA_SIGINFO) {
+        old_sig_handler.sa_sigaction(sig, info, NULL);
+      } else {
+        old_sig_handler.sa_handler(sig);
+      }
+    }
+  }
+}
 
 void processx__setup_sigchld() {
   struct sigaction action;
+  struct sigaction old;
   memset(&action, 0, sizeof(action));
   action.sa_sigaction = processx__sigchld_callback;
   action.sa_flags = SA_SIGINFO | SA_RESTART | SA_NOCLDSTOP;
-  sigaction(SIGCHLD, &action, /* oldact= */ NULL);
+  sigaction(SIGCHLD, &action, &old);
+  if (old.sa_sigaction != processx__sigchld_callback) {
+    memcpy(&old_sig_handler, &old, sizeof(old));
+  }
 }
 
 void processx__remove_sigchld() {
   struct sigaction action;
   memset(&action, 0, sizeof(action));
   action.sa_handler = SIG_DFL;
-  sigaction(SIGCHLD, &action, /* oldact= */ NULL);
+  sigaction(SIGCHLD, &action, &old_sig_handler);
+  memset(&old_sig_handler, 0, sizeof(old_sig_handler));
 }
 
 void processx__block_sigchld() {
@@ -91,7 +124,7 @@ void processx__block_sigchld() {
   sigemptyset(&blockMask);
   sigaddset(&blockMask, SIGCHLD);
   if (sigprocmask(SIG_BLOCK, &blockMask, NULL) == -1) {
-    error("processx error setting up signal handlers");
+    R_THROW_ERROR("processx error setting up signal handlers");
   }
 }
 
@@ -100,6 +133,6 @@ void processx__unblock_sigchld() {
   sigemptyset(&unblockMask);
   sigaddset(&unblockMask, SIGCHLD);
   if (sigprocmask(SIG_UNBLOCK, &unblockMask, NULL) == -1) {
-    error("processx error setting up signal handlers");
+    R_THROW_ERROR("processx error setting up signal handlers");
   }
 }
