@@ -1,13 +1,11 @@
 
-context("errors")
-
 test_that("run() prints stderr if echo = FALSE", {
   px <- get_tool("px")
   err <- tryCatch(
     run(px, c("outln", "nopppp", "errln", "bad", "errln", "foobar",
               "return", "2")),
     error = function(e) e)
-  expect_match(conditionMessage(err), "foobar")
+  expect_true(any(grepl("foobar", format(err))))
   expect_false(any(grepl("nopppp", conditionMessage(err))))
 })
 
@@ -27,17 +25,36 @@ test_that("run() handles stderr_to_stdout = TRUE properly", {
     run(px, c("outln", "nopppp", "errln", "bad", "errln", "foobar",
               "return", "2"), stderr_to_stdout = TRUE),
     error = function(e) e)
-  expect_match(conditionMessage(err), "foobar")
-  expect_match(conditionMessage(err), "nopppp")
+  expect_true(any(grepl("foobar", format(err))))
+  expect_true(any(grepl("nopppp", format(err))))
 })
 
 test_that("run() only prints the last 10 lines of stderr", {
   px <- get_tool("px")
   args <- rbind("errln", paste0("foobar", 1:11, "--"))
-  err <- tryCatch(
-    run(px, c(args, "return", "2")),
-    error = function(e) e)
-  expect_false(any(grepl("foobar1--", conditionMessage(err))))
-  expect_match(conditionMessage(err), "foobar2--")
-  expect_match(conditionMessage(err), "foobar11--")
+  withr::with_options(
+    list(rlib_interactive = TRUE),
+    ferr <- format(tryCatch(
+      run(px, c(args, "return", "2")),
+      error = function(e) e))
+  )
+  expect_false(any(grepl("foobar1--", ferr)))
+  expect_true(any(grepl("foobar2--", ferr)))
+  expect_true(any(grepl("foobar11--", ferr)))
+})
+
+test_that("prints full stderr in non-interactive mode", {
+  script <- tempfile(fileext = ".R")
+  on.exit(unlink(script, recursive = TRUE), add = TRUE)
+
+  code <- quote({
+    px <- asNamespace("processx")$get_tool("px")
+    args <- rbind("errln", paste0("foobar", 1:20, "--"))
+    processx::run(px, c(args, "return", "2"))
+  })
+  cat(deparse(code), file = script, sep = "\n")
+
+  out <- callr::rscript(script, fail_on_status = FALSE, show = FALSE)
+  expect_match(out$stderr, "foobar1--")
+  expect_match(out$stderr, "foobar20--")
 })
